package net.hostsharing.hsadminng.hs.hosting.asset.validators;

import net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAsset;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import static net.hostsharing.hsadminng.hs.hosting.asset.HsHostingAssetType.DOMAIN_SETUP;
import static net.hostsharing.hsadminng.hs.hosting.asset.validators.HsDomainHttpSetupHostingAssetValidator.SUBDOMAIN_NAME_REGEX;

class HsDomainSetupHostingAssetValidator extends HostingAssetEntityValidator {

    public static final String FQDN_REGEX = "^((?!-)[A-Za-z0-9-]{1,63}(?<!-)\\.)+[A-Za-z]{2,12}";
    public static final String DOMAIN_NAME_PROPERTY_NAME = "domainName";

    HsDomainSetupHostingAssetValidator() {
        super(  DOMAIN_SETUP,
                AlarmContact.isOptional(),

                NO_EXTRA_PROPERTIES);
    }

//    @Override
//    public List<String> validateEntity(final HsHostingAsset assetEntity) {
//        // TODO.impl: for newly created entities, check the permission of setting up a domain
//        //
//        // allow if
//        //  - user has Admin/Agent-role for all its sub-domains and the direct parent-Domain which are set up at at Hostsharing
//        //  - domain has DNS zone with TXT record approval
//        //  - parent-domain has DNS zone with TXT record approval
//        //
//        // TXT-Record check:
//        // new InitialDirContext().getAttributes("dns:_netblocks.google.com", new String[] { "TXT"}).get("TXT").getAll();
//        final var violations = new ArrayList<String>();
//        if ( assetEntity.getBookingItem() != null ) {
//            final var bookingItemDomainName = assetEntity .getDirectValue(DOMAIN_NAME_PROPERTY_NAME, String.class);
//            if (  bookingItemDomainName ) {
//                violations.add("'" + bookingItem.toShortString() + ".resources." + DOMAIN_NAME_PROPERTY_NAME + "' = '" + domainName + "' is a forbidden Hostsharing domain name");
//            }
//        }
//        violations.addAll(super.validateEntity(assetEntity));
//        return violations;
//    }

    @Override
    public List<String> validateEntity(final HsHostingAsset assetEntity) {

        final var violations = new ArrayList<String>();
        final var result = new Dns(assetEntity.getIdentifier()).fetchRecordsOfType("TXT");
        switch ( result.status() ) {
            case Dns.Status.SUCCESS:
                final var hash = assetEntity.getBookingItem().getDirectValue("verificationPassphrase", String.class);
                final var found = result.records().stream().filter(r -> r.contains("Hostsharing-domain-setup-verification-code=" + hash)).findAny();
                if (found.isEmpty()) {
                    violations.add("[DNS] no TXT record 'Hostsharing-domain-setup-verification=...' with valid hash found for domain name '" + assetEntity.getIdentifier() + "'");
                }
                break;

            case Dns.Status.NAME_NOT_FOUND:
                // no DNS verification necessary / FIXME: at least if the superdomain is at registrar level
                break;

            case Dns.Status.INVALID_NAME:
                violations.add("[DNS] invalid domain name '" + assetEntity.getIdentifier() + "'");
                break;

                case Dns.Status.SERVICE_UNAVAILABLE:
            case Dns.Status.UNKNOWN_FAILURE:
                violations.add("[DNS] lookup failed for domain name '" + assetEntity.getIdentifier() + "': " + result.exception());
                break;
        }


        violations.addAll(super.validateEntity(assetEntity));
        return violations;
    }

    @Override
    protected Pattern identifierPattern(final HsHostingAsset assetEntity) {
        if ( assetEntity.getBookingItem() != null ) {
            final var bookingItemDomainName = assetEntity.getBookingItem().getDirectValue(DOMAIN_NAME_PROPERTY_NAME, String.class);
            return Pattern.compile(bookingItemDomainName, Pattern.CASE_INSENSITIVE|Pattern.LITERAL);
        }
        final var parentDomainName = assetEntity.getParentAsset().getIdentifier();
        return Pattern.compile(SUBDOMAIN_NAME_REGEX + "\\." + parentDomainName.replace(".", "\\."), Pattern.CASE_INSENSITIVE);
    }
}
